<?php 
/**
 * @package    Proxim
 * @author     Davison Pro <davisonpro.coder@gmail.com | https://davisonpro.dev>
 * @copyright  2019 Proxim
 * @version    1.0.0
 * @since      File available since Release 1.0.0
 */

namespace Proxim;

use Db;
use Proxim\Database\DbQuery;
use Proxim\Route;
use Proxim\Configuration;
use Proxim\Tools;
use Proxim\Validate;
use Proxim\Currency;
use Proxim\Mail;
use Proxim\Site;
use Proxim\ePayments;
use Proxim\User\Employee;
use Proxim\Order\Order;
use Proxim\User\Customer;
use Proxim\Order\PayPalPayment;
use Proxim\Order\AdditionalService;
use Proxim\Util\DateUtils;
use Proxim\Util\ArrayUtils;

class SecureTradingCheckout extends Route {

    public function verifyPayment() {
		$app = $this->app;
        $payload = $app->request->post();
		$params = $app->request->get();
        $user = $app->container->user;

        $options = array();

        $handle = ArrayUtils::get($params, 'handle');

        $transactionreference = ArrayUtils::get($payload, 'transactionreference');
        $orderreference = ArrayUtils::get($payload, 'orderreference');
        $settlestatus = ArrayUtils::get($payload, 'settlestatus');
        $errorcode = ArrayUtils::get($payload, 'errorcode');

        switch( $handle ) {
            case "order":
                if( $orderreference && $settlestatus == "0" ){
                    $order =  new Order( (int) $orderreference );
                    if(!Validate::isLoadedObject($order)) {
                        return $app->response([
                            'success' => false
                        ]);
                    }
        
                    $paypalPayment = new PayPalPayment();
                    $paypalPayment->node_id = $order->id;
                    $paypalPayment->amount = $totalAmount;
                    $paypalPayment->status_id = PayPalPayment::STATE_COMPLETE;
                    $paypalPayment->update();
                    
                    $options['currency'] = array(
                        'symbol' => $order->currency->symbol,
                        'rate' => $order->currency->conversion_rate
                    );
        
                    $order->total_paid = $order->total_paid + $totalAmount;
        
                    if ( 
                        $order->status_id == Order::FAKE ||
                        $order->status_id == Order::FREE_INQUIRY ||
                        $order->status_id == Order::INQUIRY
                    ) {
                        $customer_deadline = time() + ($order->hrs_customer*60*60);
                        $writer_deadline = time() + ($order->hrs_writer*60*60);
                        $order->customer_deadline = DateUtils::date($customer_deadline);
                        $order->writer_deadline = DateUtils::date($writer_deadline);
        
                        if ($order->progressive_delivery) {
                            $order->status_id = Order::NEW_PAID;
                        } else {
                            $order->status_id = Order::ORDER_PUBLISHED;
                        }
        
                        $order->is_paid = 1;
                        $order->paid_at = DateUtils::now();
                    }
        
                    $order->update();
        
                    $customer = new Customer( (int) $order->customer_id );
                    if(Validate::isLoadedObject($customer)) {
                        $customer->total_spent = $customer->total_spent + $totalAmount;
                        $customer->update();
        
                        /** process affiliates */
                        if ( $customer->referrer_id && !$customer->affiliate_used ) {
                            $app->processAffiliates( $customer->id, $customer->referrer_id );
                        }
        
                        $site = new Site( (int) $customer->site_id );
                        if( Validate::isLoadedObject($site) ) {
                            $site->orders = $site->orders+1;       
                            $site->payments = $site->payments + $totalAmount;
                            $site->update();
                        }
        
                        $mailParams = array(
                            'order_id' => $order->id,
                            'title' => $order->title,
                            'pages' => $order->pages,
                            'payment_method' => Tools::ucfirst($order->payment_method),
                            'paid_at' => DateUtils::date( strtotime($order->paid_at) , null, 'M jS, Y \\a\\t g:i A'),
                            'paid_amount' => formatPrice($totalAmount, $options),
                            'type' => 'order',
                            'admin_site_name' => Configuration::get('SITE_NAME', PROX_ADMIN_SITE_ID, null),
                            'admin_order_url' => Configuration::get('SITE_DOMAIN', PROX_ADMIN_SITE_ID, $app->base_uri) . '/order/' . (int) $order->id,
                            'order_url' => $app->base_uri . '/dashboard/order/' . (int) $order->id,
                        );
        
                        try {
                            Mail::Send(
                                'payment',
                                'Payment confirmation for order #' . $order->id,
                                $mailParams,
                                $customer->email
                            );
                        } catch(\Exception $e) {
        
                        }
        
                        $receivers = array();
                        $receivers[] = Configuration::get('SITE_EMAIL', PROX_ADMIN_SITE_ID, null);
        
                        $sql = new DbQuery();
                        $sql->select('e.email');
                        $sql->from('employee', 'e');
                        $sql->where('employee_group = ' . (int) Employee::GROUP_WRITER);
                        $sql->where('is_started = 1');
                        $sql->where('is_banned = 0');
                        $result = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);
        
                        foreach ($result as $writer) {
                            if( ArrayUtils::has($writer, 'email') && Validate::isEmail(ArrayUtils::get($writer, 'email')) ) {
                                $receivers[] = ArrayUtils::get($writer, 'email');
                            }
                        }
        
                        foreach ($receivers as $receiver) {
                            try {
                                Mail::Send(
                                    'new_order',
                                    'New Order #' . $order->id . ' available',
                                    $mailParams,
                                    $receiver,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    null,
                                    PROX_ADMIN_SITE_ID
                                );
                            } catch(\Exception $e) {
                            }
                        }
                    }
                
                }
                break;
            
            case 'additional-payment':

                if( $orderreference && $settlestatus == "0" ){
                    $additionalService = new AdditionalService( (int) $orderreference );
                    if (!Validate::isLoadedObject($additionalService)) {
                        return $app->response([
                            'success' => false
                        ]);
                    }

                    $order = new Order( (int) $additionalService->order_id );
                    if (!Validate::isLoadedObject($order)) {
                        return $app->response([
                            'success' => false
                        ]);
                    }

                    if (!$additionalService->is_paid) {
                        // pages
                        if($additionalService->pages) $order->pages = $order->pages + $additionalService->pages;

                        // charts
                        if($additionalService->charts) $order->charts = $additionalService->charts + $additionalService->charts;

                        // slides
                        if($additionalService->slides) $order->slides = $additionalService->slides + $additionalService->slides;

                        // writer samples
                        if($additionalService->writer_samples) $order->writer_samples = 1;

                        // used sources
                        if($additionalService->used_sources) $order->used_sources = 1;

                        // smart paper
                        if($additionalService->smart_paper) $order->smart_paper = 1;

                        // expert proofreading
                        if($additionalService->expert_proofreading) $order->expert_proofreading = 1;

                        // progressive delivery
                        if($additionalService->progressive_delivery) $order->progressive_delivery = 1;

                        // writer category 
                        if($additionalService->writer_category_id) $order->writer_category_id = $additionalService->writer_category_id;

                        // shorten deadline
                        if($additionalService->tariff_id) {
                            $tariff = ArrayUtils::get($globals['deadlines'], $additionalService->tariff_id);
                            if ( $tariff ) {
                                $price_per_page = (float) ArrayUtils::get($tariff, 'pricePerPage');
                                $hrs_customer = (int) ArrayUtils::get($tariff, 'hrs');
                                $hrs_writer = (int) ArrayUtils::get($tariff, 'hrsWriter');
                                $customer_deadline = time() + ($hrs_customer*60*60);
                                $writer_deadline = time() + ($hrs_writer*60*60);

                                $order->tariff_id = $additionalService->tariff_id;
                                $order->price_per_page = $price_per_page;
                                $order->hrs_customer = $hrs_customer;
                                $order->hrs_writer = $hrs_writer;
                                $order->customer_deadline = DateUtils::date($customer_deadline);
                                $order->writer_deadline = DateUtils::date($writer_deadline);
                            }
                        } 

                        $order->update();

                        $additionalService->is_paid = 1;
                        $additionalService->paid_at = DateUtils::now();
                        $additionalService->update();

                        $customer = new Customer( (int) $order->customer_id );
                        if(Validate::isLoadedObject($customer)) {
                            $customer->total_spent = $customer->total_spent + $totalAmount;
                            $customer->update();

                            $site = new Site( (int) $customer->site_id );
                            if( Validate::isLoadedObject($site) ) {
                                $site->payments = $site->payments + $totalAmount;
                                $site->update();
                            }
                        }
                    }
                }
                
                break;
            
        }

        
    }
    
}