<?php
/**
 * @package    Proxim
 * @author     Davison Pro <davisonpro.coder@gmail.com | https://davisonpro.dev>
 * @copyright  2019 Proxim
 * @version    1.0.0
 * @since      File available since Release 1.0.0
 */

use Proxim\Configuration;
use Proxim\Database\DbQuery;
use Proxim\Module\Module;
use Proxim\Pager;
use Proxim\PrePaper\PrePaper;
use Proxim\Presenter\PrePaper\PrePaperPresenter;
use Proxim\Tools;
use Proxim\Util\ArrayUtils;
use Proxim\Util\DateUtils;
use Proxim\Validate;

require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'prox-bootstrap.php';

$smarty = $app->container->smarty;

$params = Tools::getAllValues();
$do = ArrayUtils::get($params, 'do', false);

if(!Module::getInstanceByName('prepapers')) {
    $app->setTemplate('404');
    $app->display();
}

switch( $do ) {
    case 'listPapers':
        $app->setVars([
            'page' => [
                'title' => Configuration::get('PREWRITTEN_PAPERS_TITLE') . ' | ' . Configuration::get('SITE_NAME'),
                'description' => Configuration::get('PREWRITTEN_PAPERS_DESCRIPTION')
            ]
        ]);

        $search_query = ArrayUtils::get($params, 'q', false);
        $academicLevelId = ArrayUtils::get($params, 'academicLevel', false);
        $disciplineId = ArrayUtils::get($params, 'discipline', false);
        $paperTypeId = ArrayUtils::get($params, 'paperType', false);

        $is_search = false;

        $selected_page = ArrayUtils::has($params, 'page') ? (int) ArrayUtils::get($params, 'page') : 1;

        $sql = new DbQuery();
        $sql->select('p.prepaper_id');
        $sql->from('prepaper', 'p');

        if($academicLevelId) $sql->where('p.academic_level_id = ' . (int) $academicLevelId);
        if($disciplineId) $sql->where('p.topic_category_id = ' . (int) $disciplineId);
        if($paperTypeId) $sql->where('p.paper_type_id = ' . (int) $paperTypeId);

        if ($search_query) {
            $is_search = true;
            $sql->where('
                p.title LIKE \'%' . pSQL($search_query) . '%\' OR
                p.question LIKE \'%' . pSQL($search_query) . '%\'
            ');
        }

        $sql->orderBy('p.prepaper_id DESC');

        $result = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);
        $total_prepapers = count($result);

        $prepapers = array();

        if( $total_prepapers > 0) {
            $params['total_items'] = $total_prepapers;
            $params['selected_page'] = $selected_page;
            $params['items_per_page'] = Configuration::get('MAX_RESULTS', null, 10)*2;
            $params['url'] = $app->base_uri ."/papers?page=%s";

            $pager = new Pager( $params );
            $limit_query = $pager->getLimitSql();

            $result = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql . $limit_query);

            $prepaperPresenter = new PrePaperPresenter();

            foreach( $result as $prepaper ) {
                $prepaper = new PrePaper($prepaper['prepaper_id']);
                if( Validate::isLoadedObject($prepaper) ) {
                    $prepapers[] = $prepaperPresenter->present( $prepaper );
                }
            }

            $smarty->assign([
                'pager' => $pager->getPager()
            ]);
        }

        $smarty->assign([
            'is_search' => $is_search,
            'prepapers' => $prepapers,
            'total_prepapers' => $total_prepapers
        ]);

        $app->setTemplate('prepapers/list');
        $app->display();

    case 'showPaper':
        $slug = ArrayUtils::get($params, 'slug', false);

        $cookie = $app->container->cookie;
        $user = $app->container->user;


        $prepaper = new PrePaper();
        $prepaper = $prepaper->getBySlug( $slug );
        if(!Validate::isLoadedObject($prepaper)) {
            $app->setTemplate('404');
            $app->display();
        }

        $app->setVars([
            'page' => [
                'title' => $prepaper->title . ' | ' . Configuration::get('SITE_NAME'),
                'description' => Configuration::get('PREWRITTEN_PAPERS_DESCRIPTION')
            ],
        ]);

        if(!isset($cookie->viewed_prepapers)) {
            $cookie->viewed_prepapers = null;
        }

        $viewed_prepapers = $cookie->viewed_prepapers;
        // get prepaper ids
        $viewed_prepapers = explode(',', $viewed_prepapers);
        if(empty($viewed_prepapers)) {
            $viewed_prepapers = array();
        }

        // check if prepaper is in array
        if(!in_array($prepaper->id, $viewed_prepapers)) {
            $viewed_prepapers[] = $prepaper->id;
            $prepaper->views = $prepaper->views+1;
            $prepaper->update();
        }

        $cookie->viewed_prepapers = implode(",", $viewed_prepapers);

        $prepaperPresenter = new PrePaperPresenter();

        $smarty->assign([
            'prepaper' => $prepaperPresenter->present( $prepaper )
        ]);

        $app->setTemplate('prepapers/paper');
        $app->display();
        break;

    case 'sitemap':
        header("Content-type: text/xml");

        $sql = new DbQuery();
        $sql->select('p.*');
        $sql->from('prepaper', 'p');
        $sql->orderBy('p.prepaper_id DESC');

        $result = Db::getInstance(PROX_USE_SQL_SLAVE)->executeS($sql);
        $total_prepapers = count($result);

        $prepapers = array();

        if($total_prepapers > 0) {
            foreach( $result as $prepaper ) {
                $prepapers[] = array(
                    'loc' => $app->base_uri . '/pre-paper/' . $prepaper['slug'],
                    'lastmod' => DateUtils::convertToISOFormat($prepaper['date_upd']),
                    'priority' => PrePapers::PRIORITY,
                    'changefreq' => PrePapers::CHANGEFREQ
                );
            }
        }

        $smarty->assign([
            'xls_url' => $app->base_uri . '/prox-modules/prepapers/sitemap.xsl',
            'prepapers' => $prepapers
        ]);

        $app->setTemplate('prepapers/sitemap');
        $app->display();
        break;

    default:
        $app->setTemplate('404');
        $app->display();
        break;
}